# Example of how to manually extract incubation bouts from interactive plots of raw data: R-CODE and DATA 

{# General information
# The script runs with R (Version 3.1.1; 2014-07-10) and packages plyr (Version 1.8.1), XLConnect (Version 0.2-9), utilsMPIO (Version 0.0.25), sp (Version 1.0-15), rgdal (Version 0.8-16), tools (Version 3.1.1) and lattice (Version 0.20-29)
# --------------------------------------------------------------------------------------------------------
# Questions can be directed to: Martin Bulla (bulla.mar@gmail.com)
# --------------------------------------------------------------------------------------------------------
  
  # Data collection and how the individual variables were derived is described in:
			#Steiger, S.S., et al., When the sun never sets: diverse activity rhythms under continuous daylight in free-living arctic-breeding birds. Proceedings of the Royal Society B: Biological Sciences, 2013. 280(1764): p. 20131016-20131016.
			# Dale, J., et al., The effects of life history and sexual selection on male and female plumage colouration. Nature, 2015.
  # Data are available as Rdata file
  # Missing values are NA.
  
# --------------------------------------------------------------------------------------------------------
# For better readability the subsections of the script can be collapsed  
# --------------------------------------------------------------------------------------------------------
}
{# Description of the method	
	# 1 - data are visualized in an interactive actogram with time of day on x-axis and one panel for each day of data
	# 2 - red rectangle indicates the active field, clicking with the mouse in that field on the depicted light signal generates a data point that is automatically (via custom made function) saved in the csv file. For this data extraction I recommend, to click always on the bottom line of the red rectangle, as there is always data available due to a dummy variable ("lin") that creates continuous data at the bottom of the active panel. The data are captured only if greenish vertical bar appears and if new line of data appears in R console).  
	# 3 - to extract incubation bouts, first click in the new plot has to be start of incubation, then next click depict end of incubation and the click on the same stop start of the incubation for the other sex. If the end and start of incubation are at different times, the data will be still extracted, but the sex, logger and bird_ID will be wrong. These need to be changed manually in the csv file. Similarly, the first bout for a given plot will be always assigned to male (if no data are present in the csv file) or based on previous data. Hence, whenever a data from a new plot are extracted, at a first mouse click it is worth checking whether the sex, logger and bird_ID information is correct and if not adjust it manually. 
	# 4 - if all information from one day (panel) is extracted, right-click on the plot and choose "stop". This will activate the following day (panel) for extraction. 
	# 5 - If you wish to end extraction before going through all the rectangles, just press "escape".
	}
{# Annotations of data-files from turnstone_2009_Barrow_nest-t401_transmitter.RData 
dfr
-- contains raw data on signal strength from radio tag attached to the rump of female and male, and information about when the birds where captured and incubation stage of the nest
1. who: identifies whether the recording reffers to female, male, capture or start of hatching
2. datetime_: date and time of each recording
3. logger: unique identity of the radio tag 
4. signal_: signal strength of the radio tag
5. sex: sex of the bird (f = female, m = male)
6. nest: unique identity of the nest
7. day: datetime_ variable truncated to year-month-day format
8. time: time of day in hours
9. datetime_utc: date and time of each recording, but in UTC time
10. cols: colors assigned to "who"
--------------------------------------------------------------------------------------------------------

m
-- contains metadata for a given nest
1. sp: identifies species (RUTU = Ruddy turnstone)
2. nest: unique identity of the nest
3. year_: year of observation
4. IDfemale: unique identity of the female
5. IDmale: unique identity of the male
6. lat: latitude coordinate of the nest
7. lon: longitude coordinate of the nest
8. hatch_start: date and time when the hatching of the eggs started 
9. scinam: scientific name of the species
10. breeding_site: unique identity of the breeding site (barr = Barrow, Alaska)
11. logger: type of device used to record incubation (IT - radio tag)
12. sampling: mean incubation sampling interval in seconds
--------------------------------------------------------------------------------------------------------

s
-- contains metadata for the incubating parents
1. year_: year of capture
2. species: identifies species (RUTU = Ruddy turnstone)
3. author: identifies the author who measured the bird
4. nest: unique identity of the nest
5. caught_date_time: date and time when the bird was captured
6. recapture: was the bird capture before? (0 - no, 1 - yes)
7. sex: sex of the bird (f = female, m = male)
8. bird_ID: unique identity of the bird
9. logger: unique identity of the radio tag 
--------------------------------------------------------------------------------------------------------
}
	
{# TOOLS 
#install.packages(devtools)
#devtools::install_github('mpioSNB/utilsMPIO')
#install.packages(c("XLConnect","lattice","plyr","rgdal","grid"))
			
require(XLConnect);require(lattice);require(plyr);require(ggplot2);require(grid);require(utilsMPIO)
# specify your working directory and copy the downloaded R-file into it
	wd="C:\\Users\\mbulla\\Documents\\Dropbox\\Martinuv_cernoch\\Comparative\\Data_Scripts\\"
}

{# prepare a table for extraction of bouts - necessary only once
r =data.frame(scinam=NA, sp=NA, breeding_site=NA,year=NA,nest=NA,logger=NA, bird_ID=NA,sex=NA,lat=NA,lon=NA,sure=NA,n_ass=NA,datetime_on = NA, datetime_off=NA)
	save(r, file = paste(wd,"comparative_extraction.csv",sep=""))
	write.csv(r, file = paste(wd,"comparative_extraction.csv",sep=""), row.names=FALSE)
}

{# FUNCTIONs that visualise the data and capture the datapoints
	{# prepares panels for visualisation and panels and dataset for capturing the data from the plot
	panel.identify2 <- function(x, y = NULL, subscripts = seq_along(x), labels = subscripts, 
		n = length(x), offset = 0.5, threshold = 5, panel.args = trellis.panelArgs(), 
		date_)  {
		if (missing(x)) {
			x <- panel.args$x
			y <- panel.args$y
			if (missing(subscripts) && !is.null(panel.args$subscripts)) 
				subscripts <- panel.args$subscripts
		}
		xy <- xy.coords(x, y, recycle = TRUE)
		x <- xy$x
		y <- xy$y
		px <- convertX(unit(x, "native"), "points", TRUE)
		py <- convertY(unit(y, "native"), "points", TRUE)
		labels <- as.character(labels)
		if (length(labels) > length(subscripts)) 
			labels <- labels[subscripts]
		count <- 0
		while (count < n) {
			ll <- grid.locator(unit = "points")
			if (is.null(ll)) 
				break
			lx <- convertX(ll$x, "points", TRUE)
			ly <- convertY(ll$y, "points", TRUE)
			pdists <- sqrt((px - lx)^2 + (py - ly)^2)
			if (min(pdists, na.rm = TRUE) > threshold){ 
				flush.console()
				cat("no observations within ", threshold, " points")
				flush.console()
				}else {
				w <- which.min(pdists)
					pos <- lattice:::getTextPosition(x = lx - px[w], y = ly - py[w])
					# ltext(x[w], y[w], labels[w], pos = pos, offset = offset, cex = .5, col = 2, alpha = 0.5)
					lpoints(x = x[w], y = y[w], pch = "|", col = "darkolivegreen4", cex = 5, alpha = .8)
									
				datetime_ = as.character(as.POSIXct(as.POSIXlt(as.Date(date_)),tz="UTC") + x[w] * 3600)
				
				count <- count + 1
				
				
				r<-read.csv(paste(wd,'comparative_extraction.csv',sep=""), header=TRUE, stringsAsFactors=FALSE)
				r$datetime_on=as.character(r$datetime_on)
				r$datetime_off=as.character(r$datetime_off)
				on = r$datetime_on[nrow(r)]
				off = r$datetime_off[nrow(r)]
			}	
			{# decides whether to create a new line with start of incubation bout (datetime_on) or whether to add end of incubation bout (datetime_off) into the existing line		
				if(is.na(on) & is.na(off)) {
					r = data.frame(scinam=gsub(" ","_",tolower(m$scinam[1])), sp=m$sp[1], breeding_site=m$breeding_site, year=m$year_[1],method=m$logger[1],sampling=m$sampling[1],nest=m$nest[1],logger=s$logger[1], bird_ID=s$bird_ID[1],sex=s$sex[1],lat=m$lat[1],lon=m$lon[1],sure="y",n_ass="n", datetime_on = datetime_, datetime_off="2000-01-01")
					}else{
				
					if(!is.na(on) & off=="2000-01-01") {r$datetime_off[nrow(r)]=datetime_
					}else{ 
				
					if(!is.na(on) & off!="2000-01-01"){
						sb=r$sex[nrow(r)]
						if(datetime_==off){if(sb=="f"){s_=sm}else{if(sb=="m"){s_=sf}else{if(sb=="u"){s_=su2}else{if(sb=="w"){s_=su}}}}}else{s_=s[1,]}
						res = data.frame(scinam=gsub(" ","_",tolower(m$scinam[1])), sp=m$sp[1], breeding_site=m$breeding_site[1], year=m$year_[1],method=m$logger[1],sampling=m$sampling[1],nest=m$nest[1],logger=s_$logger[1], bird_ID=s_$bird_ID[1],sex=s_$sex[1],lat=m$lat[1],lon=m$lon[1],sure="y",n_ass="n",datetime_on = datetime_, datetime_off="2000-01-01",stringsAsFactors=FALSE)
									r=rbind(r,res)}else{print("error")}
						}}
				}		
				flush.console()
				print(r[nrow(r),])
				flush.console()
				write.csv(r,file=paste(wd,'comparative_extraction.csv',sep=""), row.names=FALSE)
				
			}
    }

}
	{# visualises the data and highlights the panel in which to capture the incubation data
		actlocator2 = function(act){

									print(act)
									plocs = trellis.currentLayout()
									o = list()
									for (i in 1:nrow(plocs)) {
										trellis.focus("panel", column = 1, row = i)
										oi = panel.identify2(date_ = names(act$packet.sizes[i]))
										o = c(o, oi)
										}
									
										return(rbind, o)
									}
	}
}


{# prepare the data and plot for extraction of incubation bouts
		{# run first
			load(paste(wd,'turnstone_2009_Barrow_nest-t401_transmitter.Rdata', sep=""))
				# m - contains information about the nest
				# s - contains information about individuals
				# dfr - the raw data, that is, signal strength of the radio-tag
				
					# create separate datasets for female, male, unknown sex (data extraction function uses this)
						sf=s[s$sex=="f",] # subset for female
						sm=s[s$sex=="m",] # subset for male
						su=s[s$sex=="u",] # subset for unknown bird
			
			dfr$lin=30 # during extraction of data, we click where we think the incubation starts/ends; lin is a dummy variable that creates continuous data at the bottom of the active panel - hence clicking at the bottom of the panel ensure that the data are captured (this is also depicted by greenish vertical bar and by line that appears in R console)  
			sl = unique(dfr$day)
				
			strip.left = function(which.panel, ...) {
					LAB = format(sl[which.panel], "%b-%d")
					grid.rect(gp = gpar(fill = "lightgrey"))
					ltext(0.5, 0.5, cex = 0.8, LAB)
				}
				
			scales = list(x = list(at=c(0,6,12,18,24),labels=c('00:00','06:00','12:00','18:00','24:00') , #cex = 0.7, 
								limits = c(0,24),alternating=3), y = list(limits = c(30, 110),at =c(50,100),alternating=2, cex=0.6, tck=0.4))	
				
			panel = function(...) {
					panel.abline(v = c(1:23), col = "light grey")
					panel.abline(v=c(1:12)*2, col = "dark grey", lwd = 1.1)
					 panel.xyplot(...)
					d = dfr[which(dfr$day == sl[panel.number()]),]
					panel.xyplot(d$time[is.na(d$sex)], d$signal_[is.na(d$sex)], col = d$cols[is.na(d$sex)], type = "h", lwd=4)	
					panel.xyplot(d$time[!is.na(d$sex) & d$sex=="male"], d$signal_[!is.na(d$sex) & d$sex=="male"], col = d$cols[!is.na(d$sex) & d$sex=="male"], 
					  type = "l")	
					panel.xyplot(d$time[!is.na(d$sex) & d$sex=="female"], d$signal_[!is.na(d$sex) & d$sex=="female"], col = d$cols[!is.na(d$sex) & d$sex=="female"], 
					  type = "l")		  
					}
					
			maintitle = paste(dfr$nest[1], substr(dfr$datetime_[1],1,4), sep=", ")
				
			key = list(
						text = list(c('\u2640 signal','\u2642 signal'),cex=0.7), 
						points = list(pch = 19, col = c('#FCB42C','#535F7C')),
						text = list(c("capture","start of hatching"),cex=0.7), 
						points=list(pch=15,col=c("red",'#8C510A'),cex=0.9)
						)
			# creates the plot object
			act = xyplot(lin~ time | day, 
					data = dfr, col = c("white"), type=c("l"),
					strip.left = strip.left, layout =c(1,length(sl)),#ifelse(length(sl) > 22, 22, length(sl))), # c(1,10),#
					as.table = TRUE, 
					panel = panel, aspect = "fill", strip = FALSE, 
					distribute.type = TRUE, cex.title=0.5,main = maintitle,  scales = scales, 
					ylab='Date',
					ylab.right='Signal strength',
					xlab='Time [h]',
					key=key,
					lattice.options = list(layout.widths = list(strip.left = list(x = 3))))			
			}
		{# plot & extract
				dev.new(height=25,width=25)
				actlocator2(act)
				
				# saves the extraction plot with indicated points
					dev.print(device = png, width = 8, height = 11.6929134, units = "in", res = 300, file = paste(dw,maintitle2,"extracted.png", sep = "") )
							
		}
}

	